"""Rougail method

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

Silique (https://www.silique.fr)
Copyright (C) 2022

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from .convert import RougailConvert
from .template.base import RougailBaseTemplate
from .template.systemd import RougailSystemdTemplate
from .config import RougailConfig
from tiramisu import Config
from .update import RougailUpgrade


class Rougail:
    def __init__(self,
                 rougailconfig: RougailConfig=None,
                 ) -> None:
        if rougailconfig is None:
            rougailconfig = RougailConfig
        self.rougailconfig = rougailconfig
        self.converted = RougailConvert(self.rougailconfig)
        self.config = None

    async def get_config(self):
        if not self.config:
            tiram_obj = self.converted.save(None)
            optiondescription = {}
            exec(tiram_obj, None, optiondescription)
            self.config = await Config(optiondescription['option_0'])
        return self.config

    async def template(self,
                       type: str='base',
                       ) -> None:
        config = await self.get_config()
        if type == 'base':
            engine = RougailBaseTemplate(config, self.rougailconfig)
        else:
            engine = RougailSystemdTemplate(config, self.rougailconfig)
        await engine.instance_files()

__ALL__ = ('Rougail', 'RougailConvert', 'RougailBaseTemplate', 'RougailSystemdTemplate', 'RougailConfig', 'RougailUpgrade')
