"""Annotate value

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

Silique (https://www.silique.fr)
Copyright (C) 2022

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from rougail.annotator.variable import Walk

from rougail.i18n import _
from rougail.error import DictConsistencyError

class Annotator(Walk):  # pylint: disable=R0903
    """Annotate value
    """
    level = 70
    def __init__(self,
                 objectspace,
                 *args,
                 ) -> None:
        if not hasattr(objectspace.space, 'variables'):
            return
        self.objectspace = objectspace
        self.convert_value()
        self.add_choice_nil()

    def convert_value(self) -> None:
        """convert value
        """
        for variable in self.get_variables():
            self._convert_value(variable)

    def _convert_value(self,
                       variable,
                       ) -> None:
        # a boolean must have value, the default value is "True"
        if not hasattr(variable, 'value') and variable.type == 'boolean':
            new_value = self.objectspace.value(variable.xmlfiles)
            new_value.name = True
            new_value.type = 'boolean'
            variable.value = [new_value]
        # if the variable is mandatory and doesn't have any value
        # then the variable's mode is set to 'basic'
        # variable with default value is mandatory
        if hasattr(variable, 'value') and variable.value:
            has_value = True
            for value in variable.value:
                if value.type == 'calculation':
                    has_value = False
                    break
            if has_value and 'mandatory' not in vars(variable):
                # if has value without any calculation
                variable.mandatory = True
        if not hasattr(variable, 'value'):
            return
        if variable.value[0].type == 'calculation':
            variable.default = variable.value[0]
        elif variable.multi:
            if self.objectspace.paths.is_follower(variable):
                if variable.multi != 'submulti' and len(variable.value) != 1:
                    msg = _(f'the follower "{variable.name}" without multi attribute can only have one value')
                    raise DictConsistencyError(msg, 87, variable.xmlfiles)
            else:
                variable.default = [value.name for value in variable.value]
            if not self.objectspace.paths.is_leader(variable):
                if variable.multi == 'submulti':
                    variable.default_multi = [value.name for value in variable.value]
                else:
                    variable.default_multi = variable.value[0].name
        else:
            if len(variable.value) > 1:
                msg = _(f'the none multi variable "{variable.name}" cannot have '
                        'more than one value')
                raise DictConsistencyError(msg, 68, variable.xmlfiles)
            variable.default = variable.value[0].name
        del variable.value

    def add_choice_nil(self) -> None:
        for variable in self.get_variables():
            if variable.type != 'choice':
                continue
            is_none = False
            for choice in variable.choice:
                if choice.type == 'nil':
                    is_none = True
            if not variable.mandatory and not is_none:
                choice = self.objectspace.choice(variable.xmlfiles)
                choice.name = None
                choice.type = 'nil'
                variable.choice.append(choice)
